﻿//'*****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************/

// -------------------------------------------------------------------
// FVX Sample Program
// メインダイアログ
//
// 作成者 : R.Nagashima
//
// 履歴 : 2006/12/08 初版
// -------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
//FVXデータ構造の使用宣言
using FVDATASLib;

namespace GSBlob
{
	public partial class frnMain : Form
	{
		public frnMain()
		{
			InitializeComponent();
		}

		//処理範囲
		FvRectangle m_ExeRegion = new FvRectangle();
		// ビデオデバイスがオープンされているか
		private bool m_blnVideoOpened = false;
		// ビデオ設定ファイル名（フルパス）
		private string m_strVideoConfFilePath = "";
		// 初期ディレクトリ
		private string m_strDirectory;
		//XY基準点
		private FvPoint m_pStandatdPoint = new FvPoint();
		//2値計測実行範囲(基準画像上での座標)
		private FvRectangle m_pBlobExeRect = new FvRectangle();
		//ディスプレイ用ダイアログ
		private frmDisplayEx m_frmDisplayEx = new frmDisplayEx();
		
		private void frnMain_Load(object sender, EventArgs e)
		{
			int IntErr;
			frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
			frmLicense fLicense = new frmLicense();

			// 変数初期化
			m_frmDisplayEx.Owner = this;
			
			m_blnVideoOpened = false;
			btnGrab.Enabled = false;

			//サーチ方法
			cmbSearchMode.Items.Add("通常のパターン");   //NORMAL_PATTERN
			cmbSearchMode.Items.Add("黒い線のパターン"); //BLACK_LINE_PATTERN
			cmbSearchMode.Items.Add("白い線のパターン"); //WHITE_LINE_PATTERN
			cmbSearchMode.Items.Add("不安定パターン");   //UNSTABLE_PATTERN
			cmbSearchMode.SelectedIndex = 0;
			//サーチ精度
			cmbPrecision.Items.Add("通常精度"); //NORMAL_MODE
			cmbPrecision.Items.Add("高精度");   //HIGH_MODE
			cmbPrecision.Items.Add("超高精度"); //SUPER_MODE
			cmbPrecision.SelectedIndex = 0;
			//色設定
			cmbColor.Items.Add("BLACK");
			cmbColor.Items.Add("WHITE");
			cmbColor.SelectedIndex = 1;
			//検出順序
			cmbArrange.Items.Add("UP_TO_LEFT_RIGHT");
			cmbArrange.Items.Add("UP_TO_RIGHT_LEFT");
			cmbArrange.Items.Add("DOWN_TO_LEFT_RIGHT");
			cmbArrange.Items.Add("DOWN_TO_RIGH_LEFT");
			cmbArrange.Items.Add("LEFT_TO_UP_DOWN");
			cmbArrange.Items.Add("LEFT_TO_DOWN_UP");
			cmbArrange.Items.Add("RIGHT_TO_UP_DOWN");
			cmbArrange.Items.Add("RIGHT_TO_DOWN_UP");
			cmbArrange.SelectedIndex = 0;

			//処理範囲の初期値設定(画像の全面に設定）
			m_ExeRegion.xs = -1;
			m_ExeRegion.xe = -1;
			m_ExeRegion.ys = -1;
			m_ExeRegion.ye = -1;

			//2値計測範囲の初期設定
			m_pBlobExeRect.xs = 100;
			m_pBlobExeRect.xe = 200;
			m_pBlobExeRect.ys = 100;
			m_pBlobExeRect.ye = 200;

			//作業ディレクトリ退避
			m_strDirectory = Directory.GetCurrentDirectory();

			//使用許諾の確認
			if (fLicense.ShowDialog() == DialogResult.OK)
			{
				//ビデオ設定ファイルの選択
				if (fVideoConf.ShowDialog() == DialogResult.OK)
					m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

				m_frmDisplayEx.Show();//モーダレスで表示

				if (fVideoConf.chkNotUseCamera.Checked == false)
				{
					//ビデオ設定ファイルをコンポーネントに設定
					axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

					//デバイスオープン
					try
					{
						axFvxVideo1.OpenDevice();
					}
					catch (Exception)
					{
						axFvxVideo1.GetErrReport(out IntErr);
						MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
						return;
					}

					//VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
					//このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
					//本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
					//お使いのキャプチャボードに合わせて変更してください。
					System.Threading.Thread.Sleep(350);

					//GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
					//あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
					//（取り込みサイズとImageのサイズが違う場合エラーになります）
					axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());
					try
					{
						// 同期取込
						axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
					}
					catch (Exception)
					{
						axFvxVideo1.GetErrReport(out IntErr);
						MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
						return;
					}

					//ビデオデバイスの状態を保存
					m_blnVideoOpened = true;
					btnGrab.Enabled = true;
				}
				m_frmDisplayEx.axFvxDisplayEx1.CreateDisplay();
				m_frmDisplayEx.axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
			}
			else
			{
				//終了
				Application.Exit();
				return;
			}

		}

		//終了時
		private void frnMain_FormClosed(object sender, FormClosedEventArgs e)
		{
			if (m_blnVideoOpened)
				axFvxVideo1.CloseDevice();
		}

		//カメラ画像取込
		private void btnGrab_Click(object sender, EventArgs e)
		{
			//画像ファイル読み込みによって、画像サイズが初期化時から
			//変更されている可能性があるため、ChangeImageSizeを行います。
			//画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。
			axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());

			try
			{
				//同期取込
				axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
			}
			catch (Exception)
			{
				int IntErr;
				axFvxVideo1.GetErrReport(out IntErr);
				MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
			}

			//取り込んだ画像を画面に表示
			m_frmDisplayEx.axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());

		}

		//画像ファイル読込
		private void btnLoadImage_Click(object sender, EventArgs e)
		{
			// ダイアログにて画像ファイルを選択します｡
			openFileDialog1.InitialDirectory = m_strDirectory;
			openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
			openFileDialog1.DefaultExt = "*.bmp";
			openFileDialog1.ShowDialog();

			if (openFileDialog1.FileName.Length > 0)
			{
				//FileAccessコンポーネントを用いてファイルからImageに画像を読み込みます｡
				axFvxFileAccess1.FilePath = openFileDialog1.FileName;

				try
				{
					axFvxFileAccess1.LoadBitmap(axFvxImage1.GetOcx());
				}
				catch (Exception)
				{
					int IntErr = 0;
					axFvxFileAccess1.GetErrReport(out IntErr);
					MessageBox.Show("読込みに失敗しました｡ ErrCode= " + IntErr.ToString());
					return;
				}

				//画像タイプチェック
				if ((axFvxImage1.BitPerPixel != 8) || (axFvxImage1.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray))
				{
					MessageBox.Show("8bitグレイの画像ではありません｡");
					axFvxImage1.BitPerPixel = 8;
					axFvxImage1.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
					axFvxImage1.ImageWidth = 512;
					axFvxImage1.ImageHeight = 480;
				}
				m_frmDisplayEx.axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
			}

		}

		//画像ファイル保存
		private void btnSaveImage_Click(object sender, EventArgs e)
		{
			saveFileDialog1.InitialDirectory = m_strDirectory;
			saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
			saveFileDialog1.DefaultExt = "*.bmp";
			saveFileDialog1.ShowDialog();

			if (saveFileDialog1.FileName.Length > 0)
			{
				//FileAccessコンポーネントを使ってImageをファイルに保存します｡
				axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
				try
				{
					axFvxFileAccess1.SaveBitmap(axFvxImage1.GetOcx());
				}
				catch (Exception)
				{
					int IntErr = 0;
					axFvxFileAccess1.GetErrReport(out IntErr);
					MessageBox.Show("保存に失敗しました。　ErrCode= " + IntErr.ToString());
					return;
				}
			}

		}

		//サーチ範囲の設定
		private void btnExeRegion_Click(object sender, EventArgs e)
		{
			int IntOK = 0;
			double dblTemp = 0;

			axFvxROI1.SetRectangle(m_ExeRegion);

			// FvxROIを使って処理範囲選択ダイアログを出します｡
			axFvxROI1.UsePoint = false;
			axFvxROI1.MakeROIDialog(axFvxImage1.GetOcx(), out IntOK);

			if ((IntOK == 1) && (axFvxROI1.CheckRectangle == 1))
			{
				axFvxROI1.GetRectangle(m_ExeRegion);
				// 始点と終点が逆になっていないかチェックします｡
				// FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
				// 逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
				if (m_ExeRegion.xe < m_ExeRegion.xs)
				{
					dblTemp = m_ExeRegion.xs;
					m_ExeRegion.xs = m_ExeRegion.xe;
					m_ExeRegion.xe = dblTemp;
				}
				if (m_ExeRegion.ye < m_ExeRegion.ys)
				{
					dblTemp = m_ExeRegion.ys;
					m_ExeRegion.ys = m_ExeRegion.ye;
					m_ExeRegion.ye = dblTemp;
				}
			}
		}

		//2値計測範囲の設定
		private void button1_Click(object sender, EventArgs e)
		{
			int IntOK = 0;
			double dblTemp = 0;

			axFvxROI1.SetRectangle(m_pBlobExeRect);

			// FvxROIを使って処理範囲選択ダイアログを出します｡
			axFvxROI1.UsePoint = false;
			axFvxROI1.MakeROIDialog(axFvxImage1.GetOcx(), out IntOK);

			if ((IntOK == 1) && (axFvxROI1.CheckRectangle == 1))
			{
				axFvxROI1.GetRectangle(m_pBlobExeRect);
				// 始点と終点が逆になっていないかチェックします｡
				// FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
				// 逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
				if (m_pBlobExeRect.xe < m_pBlobExeRect.xs)
				{
					dblTemp = m_pBlobExeRect.xs;
					m_pBlobExeRect.xs = m_pBlobExeRect.xe;
					m_pBlobExeRect.xe = dblTemp;
				}
				if (m_pBlobExeRect.ye < m_pBlobExeRect.ys)
				{
					dblTemp = m_pBlobExeRect.ys;
					m_pBlobExeRect.ys = m_pBlobExeRect.ye;
					m_pBlobExeRect.ye = dblTemp;
				}
			}

		}

		//パタン登録
		private void btnSetPattern_Click(object sender, EventArgs e)
		{
			FvRectangleClass pRect = new FvRectangleClass();//パタン領域
			FvPointClass pPoint = new FvPointClass();//パタン基準点

			int IntOK = 0;
			double dblTemp = 0;

			// 登録時の初期矩形を設定
			pRect.xs = 100;
			pRect.ys = 100;
			pRect.xe = 200;
			pRect.ye = 200;
			axFvxROI2.SetRectangle(pRect);

			//FvxROIを使って処理範囲選択ダイアログを出します。
			axFvxROI2.MakeROIDialog(axFvxImage1.GetOcx(), out IntOK);

			if ((IntOK == 1) && (axFvxROI2.CheckRectangle == 1))
			{
				axFvxROI2.GetRectangle(pRect);

				//始点と終点が逆になっていないかチェックします｡
				//FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
				//逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
				if (pRect.xe < pRect.xs)
				{
					dblTemp = pRect.xs;
					pRect.xs = pRect.xe;
					pRect.xe = dblTemp;
				}
				if (pRect.ye < pRect.ys)
				{
					dblTemp = pRect.ys;
					pRect.ys = pRect.ye;
					pRect.ye = dblTemp;
				}

				if (axFvxROI2.CheckPoint == 1)
					axFvxROI2.GetPoint(pPoint);
				else
				{
					MessageBox.Show("パターン基準点が指定されませんでした。\r\nパターン矩形の中心を基準点として登録します｡", "パターン登録", MessageBoxButtons.OK, MessageBoxIcon.Error);
					pPoint.x = (pRect.xs + pRect.xe) / 2;
					pPoint.y = (pRect.ys + pRect.ye) / 2;
				}
				axFvxPattern1.MakePattern(axFvxImage1.GetOcx(), pRect, pPoint);
			}

		}

		//パタンのマスク設定
		private void btnSetPatternMask_Click(object sender, EventArgs e)
		{
			axFvxPattern1.DefMask(-1, -1);
		}

		//パタン表示
		private void btnDispPattern_Click(object sender, EventArgs e)
		{
			axFvxPattern1.Disp();
		}
		
		//パタン読込
		private void btnLoadPattern_Click(object sender, EventArgs e)
		{
			openFileDialog1.InitialDirectory = m_strDirectory;
			openFileDialog1.Filter = "pattern file(*.pat)|*.pat";
			openFileDialog1.DefaultExt = "*.pat";
			openFileDialog1.ShowDialog();

			if (openFileDialog1.FileName.Length > 0)
			{
				try
				{
					axFvxPattern1.LoadPattrenFileName = openFileDialog1.FileName;
					axFvxPattern1.Load();
				}
				catch (Exception)
				{
					int IntErr;
					axFvxPattern1.GetErrReport(out IntErr);
					MessageBox.Show("パターン読込失敗  ErrCode = " + IntErr.ToString(), "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
			}
		}

		//パタン保存
		private void btnSavePattern_Click(object sender, EventArgs e)
		{
			//パタンファイルの拡張子は特に決められていませんが、
			//"pat"を使用されることを推奨します。
			saveFileDialog1.InitialDirectory = m_strDirectory;
			saveFileDialog1.Filter = "pattern file(*.pat)|*.pat";
			saveFileDialog1.DefaultExt = "*.pat";
			saveFileDialog1.ShowDialog();
			if (saveFileDialog1.FileName.Length > 0)
			{
				try
				{
					axFvxPattern1.SavePattrenFileName = saveFileDialog1.FileName;
					axFvxPattern1.Save();
				}
				catch (Exception)
				{
					int IntErr;
					axFvxPattern1.GetErrReport(out IntErr);
					MessageBox.Show("パターン保存失敗  ErrCode = " + IntErr.ToString(), "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
			}
		}

		//2値化試行
		private void btnApplyBinLevel_Click(object sender, EventArgs e)
		{
			int intBinLevel = 0;
			if (int.TryParse(txtBinLevel.Text, out intBinLevel) == false ||  255 < intBinLevel)
				MessageBox.Show("2値化閾値は 0～255 の範囲で入力して下さい");
			else
			{
				axFvxChangeImage1.FixedLevel(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), intBinLevel);
				m_frmDisplayEx.axFvxDisplayEx1.DrawImage(axFvxImage2.GetOcx());
			}
		}

		//パラメータ設定
		private bool ApplyChanges()
		{
			int intTemp = 0;
			int intMinSize = 0;
			int intMaxSize = 0;

			//グレイサーチパラメータ----------------------------------------------------------------
			axFvxCorrelation1.SetInspectArea(m_ExeRegion);
			axFvxCorrelation1.SearchMode = (FVXCOMCORRELATIONLib.SEARCH_MODE_TYPE)cmbSearchMode.SelectedIndex;
			axFvxCorrelation1.Precision = (FVXCOMCORRELATIONLib.PRECISION_TYPE)cmbPrecision.SelectedIndex;

			//画像の状態
			if (int.TryParse(txtCondition.Text, out intTemp) == false || intTemp < 1 || intTemp > 9)
			{
				MessageBox.Show("状態は 1～9 の間で設定して下さい");
				return (false);
			}
			axFvxCorrelation1.Condition = intTemp;

			//判定閾値
			if(int.TryParse(txtThreshold.Text,out intTemp) == false || intTemp < 10 || intTemp > 99 )
			{
				MessageBox.Show("判定スコアは 10～99 の間で入力してください");
				return(false);
			}
			axFvxCorrelation1.ScoreThresh = intTemp;

			//途中判定閾値
			if (int.TryParse(txtMiddleThreshold.Text, out intTemp) == false || intTemp < 10 || intTemp > 99)
			{
				MessageBox.Show("途中判定スコアは 10～99 の間で入力してください");
				return (false);
			}
			axFvxCorrelation1.ScoreThresh2 = intTemp;

			//サーチ個数
			axFvxCorrelation1.SearchNum = 1;

			//範囲接触している物を検出するかどうか
			if(chkWindowTouchGray.Checked)
				axFvxCorrelation1.WindowTouch = FVXCOMCORRELATIONLib.WINDOW_TOUCH_TYPE.ON;
			else 
				axFvxCorrelation1.WindowTouch = FVXCOMCORRELATIONLib.WINDOW_TOUCH_TYPE.OFF;

			//2値特徴量計測パラメータ----------------------------------------------------------------
			//2値化したImageをFvxImage2に作成
			if (int.TryParse(txtBinLevel.Text, out intTemp) == false || intTemp > 255)
			{
				MessageBox.Show("2値化閾値は 0～255 の間で入力してください");
				return (false);
			}
			axFvxChangeImage1.FixedLevel(axFvxImage1.GetOcx(),axFvxImage2.GetOcx(),intTemp);

			//面積値チェック
			if (int.TryParse(txtMinSize.Text, out intMinSize) == false || intMinSize < 1)
			{
				MessageBox.Show("最小面積値は 1 以上の数値を設定して下さい");
				return (false);
			}
			if (int.TryParse(txtMaxSize.Text, out intMaxSize) == false || intMinSize >= intMaxSize)
			{
				MessageBox.Show("最小面積値＜最大面積値となるように設定して下さい");
				return (false);
			}
			//最大・最小面積設定
			axFvxBlob1.MinSize = intMinSize;
			axFvxBlob1.MaxSize = intMaxSize;

			//対象色設定
			axFvxBlob1.Color = (FVXCOMBLOBLib.COLOR_TYPE)cmbColor.SelectedIndex;

			//ラベリング（検出順序）設定
			axFvxBlob1.Arrange = (FVXCOMBLOBLib.ARRANGE_TYPE)cmbArrange.SelectedIndex;

			//処理範囲接触許可
			if (chkWindowTouchBin.Checked)
				axFvxBlob1.WindowTouch = FVXCOMBLOBLib.WINDOW_TOUCH_TYPE.ON;
			else
				axFvxBlob1.WindowTouch = FVXCOMBLOBLib.WINDOW_TOUCH_TYPE.OFF;

			return (true);
		}

		//基準位置計測
		private void btnSetStandard_Click(object sender, EventArgs e)
		{
			FvSearchResult pSearchResult = new FvSearchResult();
			int intCount;

			m_frmDisplayEx.axFvxDisplayEx1.ClearOverlay();
			m_frmDisplayEx.axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());

			//パラメータ設定
			if (!ApplyChanges()) return;

			try
			{

				//パターン設定
				axFvxCorrelation1.SetPattern(axFvxPattern1.GetOcx());

				//サーチ実行
				axFvxCorrelation1.Execute(axFvxImage1.GetOcx(), out intCount);

				//処理範囲を青で表示
				m_frmDisplayEx.axFvxDisplayEx1.PenColor = Color.Blue;
				m_frmDisplayEx.axFvxDisplayEx1.DrawData(0, m_ExeRegion);

				if (intCount > 0)
				{
					//サーチ結果を赤で表示
					axFvxCorrelation1.GetResult(0, pSearchResult);
					m_frmDisplayEx.axFvxDisplayEx1.PenColor = Color.Red;
					m_frmDisplayEx.axFvxDisplayEx1.DrawData(0, pSearchResult);

					//基準点として登録
					m_pStandatdPoint.x = pSearchResult.x;
					m_pStandatdPoint.y = pSearchResult.y;

					m_frmDisplayEx.axFvxDisplayEx1.PenColor = Color.Red;
					m_frmDisplayEx.axFvxDisplayEx1.DrawData(0, m_pBlobExeRect);//計測範囲の表示					
				}
				else
					MessageBox.Show("サーチパタンが見つかりませんでした", "サーチ失敗");
			}
			catch (System.Runtime.InteropServices.COMException)
			{
				int intErr;
				axFvxCorrelation1.GetErrReport(out intErr);
				MessageBox.Show(GetCorrelationErrMesg(intErr), "サーチエラー");
			}
		}

		//実行
		private void btnExecute_Click(object sender, EventArgs e)
		{
			FvSearchResult pSearchResult = new FvSearchResult();	//グレイサーチの結果
			FvRectangle pInpRect = new FvRectangle();				//2値計測を行う領域(アライメント後の座標)
			FvBlobResultGroup pBlobResultGroup = new FvBlobResultGroup();//2値計測結果

			double dblDiffX = 0;//ズレ量
			double dblDiffY = 0;

			double dblTime = 0;
			int intCount = 0;
			int intErr = 0;

			m_frmDisplayEx.axFvxDisplayEx1.ClearOverlay();

			//パラメータ適用
			if (!ApplyChanges()) return;

			//時間計測開始
			axFvxPerformanceCounter1.StartCount();

			//パタン設定
			axFvxCorrelation1.SetPattern(axFvxPattern1.GetOcx());

			//サーチ実行
			axFvxCorrelation1.Execute(axFvxImage1.GetOcx(), out intCount);
			axFvxCorrelation1.GetErrReport( out intErr);

			if (intErr == 0)
			{
				//処理範囲を青で表示
				m_frmDisplayEx.axFvxDisplayEx1.PenColor = Color.Purple;
				m_frmDisplayEx.axFvxDisplayEx1.DrawData(0, m_ExeRegion);

				//サーチ結果を赤で表示
				m_frmDisplayEx.axFvxDisplayEx1.PenColor = Color.Red;

				if (intCount > 0)
				{
					axFvxCorrelation1.GetResult(0, pSearchResult);
					m_frmDisplayEx.axFvxDisplayEx1.DrawData(0, pSearchResult);

					//基準位置からのズレ量を算出
					dblDiffX = pSearchResult.x - m_pStandatdPoint.x;
					dblDiffY = pSearchResult.y - m_pStandatdPoint.y;
					//2値計測範囲を移動
					pInpRect.xs = m_pBlobExeRect.xs + dblDiffX;
					pInpRect.ys = m_pBlobExeRect.ys + dblDiffY;
					pInpRect.xe = m_pBlobExeRect.xe + dblDiffX;
					pInpRect.ye = m_pBlobExeRect.ye + dblDiffY;
					axFvxBlob1.SetInspectArea(pInpRect);
					m_frmDisplayEx.axFvxDisplayEx1.DrawData(0, pInpRect);//計測範囲の表示

					//2値化処理はApplychangesの中で実行している
					m_frmDisplayEx.axFvxDisplayEx1.DrawImage(axFvxImage2.GetOcx());

					//2値計測実行
					axFvxBlob1.Execute(axFvxImage2.GetOcx(), out intCount);
					axFvxBlob1.GetErrReport(out intErr);

					if (intErr == 0)
					{
						m_frmDisplayEx.axFvxDisplayEx1.PenColor = Color.Blue;
						axFvxBlob1.GetResult(0, pBlobResultGroup);
						m_frmDisplayEx.axFvxDisplayEx1.DrawData(0, pBlobResultGroup);//重心点と外接長方形を描画

						txtResult.Text = string.Format("ズレＸ：{0:0.00}\r\nズレＹ：{1:0.00}\r\n\r\n検出ブローブ数：{2}", dblDiffX, dblDiffY, intCount);
					}
					else
					{
						MessageBox.Show(GetBlobErrMesg(intErr), "2値計測エラー");

					}
				}
				else
				{
					MessageBox.Show("サーチパタンが見つかりませんでした", "エラー");
				}
			}
			else
			{
				MessageBox.Show(GetCorrelationErrMesg(intErr), "グレイサーチエラー");
			}
			//時間計測終了
			axFvxPerformanceCounter1.StopCount(out dblTime);
			txtResult.Text = "処理時間：" + dblTime.ToString("0.00ms") + "\r\n\r\n" + txtResult.Text;
		}

		//サンプル実行
		private void btnDoSample_Click(object sender, EventArgs e)
		{
			int intErr = 0;
			//マスタ画像読込
			axFvxFileAccess1.FilePath = m_strDirectory + "/sample1.bmp";
			axFvxFileAccess1.LoadBitmap(axFvxImage1.GetOcx());
			axFvxFileAccess1.GetErrReport(out intErr);
			if(intErr != 0)
			{
				MessageBox.Show("sample1.bmpが読み込めませんでした");
				return;
			}
			m_frmDisplayEx.axFvxDisplayEx1.ClearOverlay();
			m_frmDisplayEx.axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
			MessageBox.Show("マスタ画像読込");

			//パタン読込
			axFvxPattern1.LoadPattrenFileName = m_strDirectory + "/sample1.pat";
			axFvxPattern1.Load();
			axFvxPattern1.GetErrReport(out intErr);
			if(intErr != 0)
			{
				MessageBox.Show("sample.patが読み込めませんでした");
				return;
			}
			//2値計測範囲設定
			m_pBlobExeRect.xs = 120;
			m_pBlobExeRect.ys = 270;
			m_pBlobExeRect.xe = 250;
			m_pBlobExeRect.ye = 303;

			//基準位置測定
			btnSetStandard_Click(sender, e);
			MessageBox.Show("パタン設定＆2値計測領域設定＆基準測定");

			//処理画像読込
			axFvxFileAccess1.FilePath = m_strDirectory + "/sample2.bmp";
			axFvxFileAccess1.LoadBitmap(axFvxImage1.GetOcx());
			axFvxFileAccess1.GetErrReport(out intErr);
			if(intErr != 0)
			{
				MessageBox.Show("sample2.bmpが読み込めませんでした");
				return;
			}
			m_frmDisplayEx.axFvxDisplayEx1.ClearOverlay();
			m_frmDisplayEx.axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
			MessageBox.Show("処理画像読込");

			//実行
			btnExecute_Click(sender, e);
			MessageBox.Show("アライメント＆2値計測実行");
		}

		private string GetCorrelationErrMesg(int intErr)
		{
			string Str = "";
			switch (intErr)
			{
				case 5001:
					Str = "パタンが存在していません";
					break;
				case 5005:
					Str = "パラメータが違います";
					break;
				case 5006:
					Str = "型が違います";
					break;
				case 5007:
					Str = "パタン画像が一定の為登録できません";
					break;
				case 5008:
					Str = "パタンとマスクのサイズが一致していません。";
					break;
				case 5009:
					Str = "パラメータにNULL値が指定された";
					break;
				case 5010:
					Str = "メモリの確保に失敗しました";
					break;
				case 5022:
					Str = "パタンが指定されていません";
					break;
				default:
					Str = "不明なエラー";
					break;
			}
			return (Str);
		}
		private string GetBlobErrMesg(int intErr)
		{
			string Str = "";
			switch (intErr)
			{
				case 3001:
					Str = "イメージオブジェクトにNULLが指定されています";
					break;
				case 3005:
					Str = "パラメータが違います";
					break;
				case 3006:
					Str = "引数の型が違います";
					break;
				case 3007:
					Str = "引数にNULLが指定されています";
					break;
				case 3010:
					Str = "メモリの確保に失敗しました";
					break;
				case 3011:
					Str = "最大中個数オーバーフロー";
					break;
				case 3012:
					Str = "ブローブが多すぎます";
					break;
				case 3014:
					Str = "ブローブが一つもありません";
					break;
				default:
					Str = "不明なエラー";
					break;
			}
			return (Str);
		}
	}
}